@@ -40,16 +40,14 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade-xml.h>
-#include <libgnomeui/gnome-entry.h>
 #include <gconf/gconf-client.h>
 #include <gmenu-tree.h>
 
-#define GNOME_DESKTOP_USE_UNSTABLE_API
-#include <libgnome/gnome-desktop-utils.h>
+#include <libgnome-desktop/gnome-desktop-utils.h>
 
 #include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-gtk.h>
 #include <libpanel-util/panel-keyfile.h>
 #include <libpanel-util/panel-show.h>
 
@@ -72,8 +70,7 @@ typedef struct {
 	GtkWidget        *main_box;
 	GtkWidget        *program_list_box;
 
-	GtkWidget        *gnome_entry;
-	GtkWidget        *gtk_entry;
+	GtkWidget        *combobox;
 	GtkWidget        *pixmap;
 	GtkWidget        *run_button;
 	GtkWidget        *file_button;
@@ -119,6 +116,74 @@ static PanelRunDialog *static_dialog = NULL;
 
 static void panel_run_dialog_disconnect_pixmap (PanelRunDialog *dialog);
 
+#define PANEL_RUN_HISTORY_KEY "/apps/gnome-settings/gnome-panel/history-gnome-run"
+#define PANEL_RUN_MAX_HISTORY 10
+
+static GtkTreeModel *
+_panel_run_get_recent_programs_list (void)
+{
+	GtkListStore *list;
+	GSList       *gconf_items;
+	GSList       *command;
+	int           i = 0;
+
+	list = gtk_list_store_new (1, G_TYPE_STRING);
+
+	gconf_items = gconf_client_get_list (panel_gconf_get_client (),
+					     PANEL_RUN_HISTORY_KEY,
+					     GCONF_VALUE_STRING, NULL);
+
+	for (command = gconf_items;
+	     command && i < PANEL_RUN_MAX_HISTORY;
+	     command = command->next) {
+		GtkTreeIter iter;
+		gtk_list_store_prepend (list, &iter);
+		gtk_list_store_set (list, &iter, 0, command->data, -1);
+		i++;
+	}
+
+	g_slist_free (gconf_items);
+
+	return GTK_TREE_MODEL (list);
+}
+
+static void
+_panel_run_save_recent_programs_list (GtkComboBox      *entry,
+				      char             *lastcommand)
+{
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	GSList       *gconf_items = NULL;
+	int           i = 0;
+
+	gconf_items = g_slist_prepend (gconf_items, lastcommand);
+	i++;
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (entry));
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		char *command;
+
+		do {
+			gtk_tree_model_get (model, &iter, 0, &command, -1);
+
+			if (strcmp (command, lastcommand) == 0)
+				continue;
+
+			gconf_items = g_slist_prepend (gconf_items, command);
+			i++;
+		} while (gtk_tree_model_iter_next (model, &iter) &&
+			 i < PANEL_RUN_MAX_HISTORY);
+	}
+
+	gconf_client_set_list (panel_gconf_get_client (),
+			       PANEL_RUN_HISTORY_KEY,
+			       GCONF_VALUE_STRING, gconf_items,
+			       NULL);
+
+	g_slist_free (gconf_items);
+}
+
 static void
 panel_run_dialog_destroy (PanelRunDialog *dialog)
 {
@@ -179,6 +244,16 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 	g_free (dialog);
 }
 
+static const char *
+panel_run_dialog_get_combo_text (PanelRunDialog *dialog)
+{
+	GtkWidget *entry;
+
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
+
+	return gtk_entry_get_text (GTK_ENTRY (entry));
+}
+
 static void
 panel_run_dialog_set_default_icon (PanelRunDialog *dialog, gboolean set_drag)
 {
@@ -290,35 +365,46 @@ command_is_executable (const char   *command,
 	return TRUE;
 }
 
+/*
+ * Set the DISPLAY variable, to be use by g_spawn_async.
+ */
+static void
+set_environment (gpointer display)
+{
+  g_setenv ("DISPLAY", display, TRUE);
+}
+
 static gboolean
 panel_run_dialog_launch_command (PanelRunDialog *dialog,
 				 const char     *command,
 				 const char     *locale_command)
 {
 	GdkScreen  *screen;
-	gboolean    result;	
+	gboolean    result;
 	GError     *error = NULL;
 	char      **argv;
 	int         argc;
-	
+	char    *display;
+
 	if (!command_is_executable (locale_command, &argc, &argv))
 		return FALSE;
 
-	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));	
-		
+	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)))
 		gnome_desktop_prepend_terminal_to_vector (&argc, &argv);
-		   
-	result = gdk_spawn_on_screen (screen,
-				      NULL, /* working directory */
-				      argv,
-				      NULL, /* envp */
-				      G_SPAWN_SEARCH_PATH,
-				      NULL, /* child setup func */
-				      NULL, /* user data */
-				      NULL, /* child pid */
-				      &error);
-			
+
+	display = gdk_screen_make_display_name (screen);
+
+	result = g_spawn_async (NULL, /* working directory */
+				argv,
+				NULL, /* envp */
+				G_SPAWN_SEARCH_PATH,
+				set_environment,
+				display,
+				NULL,
+				&error);
+
 	if (!result) {
 		char *primary;
 
@@ -331,9 +417,10 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 
 		g_error_free (error);
 	}
-				
+
+	g_free (display);
 	g_strfreev (argv);
-	
+
 	return result;
 }
 
@@ -346,7 +433,7 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 	char     *disk;
 	char     *scheme;	
 	
-	command = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry)));
+	command = g_strdup (panel_run_dialog_get_combo_text (dialog));
 	command = g_strchug (command);
 
 	if (!command || !command [0]) {
@@ -412,9 +499,8 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 		
 	if (result) {
 		/* only save working commands in history */
-		gnome_entry_prepend_history (GNOME_ENTRY (dialog->gnome_entry),
-					     TRUE, /* save item in history */
-					     command);
+		_panel_run_save_recent_programs_list
+			(GTK_COMBO_BOX (dialog->combobox), command);
 		
 		/* only close the dialog if we successfully showed or launched
 		 * something */
@@ -475,20 +561,21 @@ panel_run_dialog_append_file_utf8 (PanelRunDialog *dialog,
 {
 	const char *text;
 	char       *quoted, *temp;
+	GtkWidget  *entry;
 	
 	/* Don't allow filenames beginning with '-' */
 	if (!file || !file[0] || file[0] == '-')
 		return;
 	
 	quoted = quote_string (file);
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry));
-	
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
+	text = gtk_entry_get_text (GTK_ENTRY (entry));
 	if (text && text [0]) {
 		temp = g_strconcat (text, " ", quoted, NULL);
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry), temp);
+		gtk_entry_set_text (GTK_ENTRY (entry), temp);
 		g_free (temp);
 	} else
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry), quoted);
+		gtk_entry_set_text (GTK_ENTRY (entry), quoted);
 	
 	g_free (quoted);
 }
@@ -579,17 +666,13 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	GtkTreeIter   iter;
 	GtkTreeModel *model;
 	GtkTreePath  *path;
-	const char   *text;
+	char         *text;
 	char         *found_icon;
 	char         *found_name;
 	gboolean      fuzzy;
 	
 	model = GTK_TREE_MODEL (dialog->program_list_store);
 	path = gtk_tree_path_new_first ();
-	text = sure_string (gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry)));
-	found_icon = NULL;
-	found_name = NULL;
-	fuzzy = FALSE;
 	
 	if (!path || !gtk_tree_model_get_iter (model, &iter, path)) {
 		if (path)
@@ -601,6 +684,11 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 		return FALSE;
 	}
 
+	text = g_strdup (panel_run_dialog_get_combo_text (dialog));
+	found_icon = NULL;
+	found_name = NULL;
+	fuzzy = FALSE;
+
 	do {
 		char *exec = NULL;
 		char *icon = NULL;
@@ -658,6 +746,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	//FIXME update dialog->program_label
 
 	g_free (found_icon);
+	g_free (text);
 	
 	g_free (dialog->item_name);
 	dialog->item_name = found_name;
@@ -720,8 +809,8 @@ static int
 compare_applications (GMenuTreeEntry *a,
 		      GMenuTreeEntry *b)
 {
-	return g_utf8_collate (gmenu_tree_entry_get_name (a),
-			       gmenu_tree_entry_get_name (b));
+	return g_utf8_collate (gmenu_tree_entry_get_display_name (a),
+			       gmenu_tree_entry_get_display_name (b));
 }
 
 static GSList *get_all_applications_from_dir (GMenuTreeDirectory *directory,
@@ -797,6 +886,7 @@ get_all_applications (void)
 	GSList             *retval;
 
 	tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_NONE);
+	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
 
 	root = gmenu_tree_get_root_directory (tree);
 
@@ -842,7 +932,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 		next = l->next;
 
-		entry_name = gmenu_tree_entry_get_name (entry);
+		entry_name = gmenu_tree_entry_get_display_name (entry);
 		if (prev_name && entry_name && strcmp (entry_name, prev_name) == 0) {
 			gmenu_tree_item_unref (entry);
 
@@ -861,7 +951,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 		gtk_list_store_set (dialog->program_list_store, &iter,
 				    COLUMN_ICON,      NULL,
 				    COLUMN_ICON_FILE, gmenu_tree_entry_get_icon (entry),
-				    COLUMN_NAME,      gmenu_tree_entry_get_name (entry),
+				    COLUMN_NAME,      gmenu_tree_entry_get_display_name (entry),
 				    COLUMN_COMMENT,   gmenu_tree_entry_get_comment (entry),
 				    COLUMN_EXEC,      gmenu_tree_entry_get_exec (entry),
 				    COLUMN_PATH,      gmenu_tree_entry_get_desktop_file_path (entry),
@@ -964,7 +1054,8 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	char         *path, *stripped;
 	gboolean      terminal;
 	GKeyFile     *key_file;
-		
+	GtkWidget    *entry;
+
 	if (!gtk_tree_selection_get_selected (selection, &filter_model,
 					      &filter_iter))
 		return;
@@ -997,21 +1088,20 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	if (dialog->item_name)
 		g_free (dialog->item_name);
 	dialog->item_name = NULL;
-	
+
 	/* Order is important here. We have to set the text first so that the
 	 * drag source is enabled, otherwise the drag icon can't be set by
 	 * panel_run_dialog_set_icon.
 	 */
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
 	temp = panel_key_file_get_string (key_file, "Exec");
 	if (temp) {
 		stripped = remove_parameters (temp);
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry),
-				    stripped);
+		gtk_entry_set_text (GTK_ENTRY (entry), stripped);
 		g_free (stripped);
 	} else {
 		temp = panel_key_file_get_string (key_file, "URL");
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry),
-				    sure_string (temp));
+		gtk_entry_set_text (GTK_ENTRY (entry), sure_string (temp));
 	}
 	g_free (temp);
 
@@ -1053,14 +1143,14 @@ program_list_selection_activated (GtkTreeView       *view,
 
 static void
 panel_run_dialog_setup_program_list (PanelRunDialog *dialog,
-				     GladeXML       *gui)
+				     GtkBuilder     *gui)
 {
 	GtkTreeSelection *selection;
 	
-	dialog->program_list = glade_xml_get_widget (gui, "program_list");
-	dialog->program_list_box = glade_xml_get_widget (gui, "program_list_box");
-	dialog->program_label = glade_xml_get_widget (gui, "program_label");
-	dialog->main_box = glade_xml_get_widget (gui, "main_box");
+	dialog->program_list = PANEL_GTK_BUILDER_GET (gui, "program_list");
+	dialog->program_list_box = PANEL_GTK_BUILDER_GET (gui, "program_list_box");
+	dialog->program_label = PANEL_GTK_BUILDER_GET (gui, "program_label");
+	dialog->main_box = PANEL_GTK_BUILDER_GET (gui, "main_box");
 	
 	/* Ref the box so it doesn't get destroyed when it is
 	 * removed from the visible area of the dialog box.
@@ -1091,12 +1181,15 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 				 gboolean        show_list)
 {
 	if (!panel_profile_get_enable_program_list ()) {
-		if (dialog->list_expander->parent)
-			gtk_container_remove (GTK_CONTAINER (dialog->list_expander->parent),
+		GtkWidget *parent;
+
+		parent = gtk_widget_get_parent (dialog->list_expander);
+		if (parent)
+			gtk_container_remove (GTK_CONTAINER (parent),
 					      dialog->list_expander);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
-                gtk_widget_grab_focus (dialog->gtk_entry);
+                gtk_widget_grab_focus (dialog->combobox);
 		
 	} else if (show_list) {
 		gtk_window_resize (GTK_WINDOW (dialog->run_dialog), 100, 300);
@@ -1105,7 +1198,7 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 		
         } else if (!show_list) {
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
-                gtk_widget_grab_focus (dialog->gtk_entry);
+                gtk_widget_grab_focus (dialog->combobox);
         }
 }
 
@@ -1128,12 +1221,12 @@ list_expander_toggled (GtkExpander    *expander,
 
 static void
 panel_run_dialog_setup_list_expander (PanelRunDialog *dialog,
-				      GladeXML       *gui)
+				      GtkBuilder     *gui)
 {
 	GConfClient *client;
 	const char *key;
 	
-	dialog->list_expander = glade_xml_get_widget (gui, "list_expander");
+	dialog->list_expander = PANEL_GTK_BUILDER_GET (gui, "list_expander");
 
 	if (panel_profile_get_enable_program_list ()) {
 		gtk_expander_set_expanded (GTK_EXPANDER (dialog->list_expander),
@@ -1174,7 +1267,7 @@ file_button_browse_response (GtkWidget      *chooser,
 
 	gtk_widget_destroy (chooser);
  
- 	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_grab_focus (dialog->combobox);
 }
 
 static void
@@ -1204,9 +1297,9 @@ file_button_clicked (GtkButton      *button,
 
 static void
 panel_run_dialog_setup_file_button (PanelRunDialog *dialog,
-				    GladeXML       *gui)
+				    GtkBuilder     *gui)
 {
-	dialog->file_button = glade_xml_get_widget (gui, "file_button");
+	dialog->file_button = PANEL_GTK_BUILDER_GET (gui, "file_button");
 		
         g_signal_connect (dialog->file_button, "clicked",
 			  G_CALLBACK (file_button_clicked),
@@ -1438,7 +1531,7 @@ entry_event (GtkEditable    *entry,
 		return FALSE;
 
 	/* tab completion */
-	if (event->keyval == GDK_Tab) {
+	if (event->keyval == GDK_KEY_Tab) {
 		gtk_editable_get_selection_bounds (entry, &pos, &tmp);
 
 		if (dialog->completion_started &&
@@ -1498,10 +1591,10 @@ entry_event (GtkEditable    *entry,
 
 			prefix = g_strconcat (temp, nprefix, NULL);
 
-			g_signal_handler_block (dialog->gtk_entry,
+			g_signal_handler_block (dialog->combobox,
 						dialog->changed_id);
 			gtk_editable_delete_text (entry, 0, -1);
-			g_signal_handler_unblock (dialog->gtk_entry,
+			g_signal_handler_unblock (dialog->combobox,
 						  dialog->changed_id);
 
 			gtk_editable_insert_text (entry,
@@ -1527,14 +1620,14 @@ entry_event (GtkEditable    *entry,
 }
 
 static void
-entry_changed (GtkEntry       *entry,
-	       PanelRunDialog *dialog)
+combobox_changed (GtkComboBox    *combobox,
+		  PanelRunDialog *dialog)
 {
-	const char *text;
-	const char *start;
-	char       *msg;
+	char *text;
+	char *start;
+	char *msg;
 
-	text = gtk_entry_get_text (entry);
+        text = g_strdup (panel_run_dialog_get_combo_text (dialog));
 
 	start = text;
 	while (*start != '\0' && g_ascii_isspace (*start))
@@ -1554,6 +1647,8 @@ entry_changed (GtkEntry       *entry,
 
 	/* desensitize run button if no text entered */
 	if (!start || !start [0]) {
+		g_free (text);
+
 		gtk_widget_set_sensitive (dialog->run_button, FALSE);
 		gtk_drag_source_unset (dialog->run_dialog);
 
@@ -1611,6 +1706,8 @@ entry_changed (GtkEntry       *entry,
 			g_idle_add_full (G_PRIORITY_LOW,
 					 (GSourceFunc) panel_run_dialog_find_command_idle,
 					 dialog, NULL);
+
+	g_free (text);
 }
 
 static void
@@ -1627,14 +1724,14 @@ entry_drag_data_received (GtkEditable      *entry,
 	char  *file;
 	int    i;
 
-        if (selection_data->format != 8 || selection_data->length == 0) {
+        if (gtk_selection_data_get_format (selection_data) != 8 || gtk_selection_data_get_length (selection_data) == 0) {
         	g_warning (_("URI list dropped on run dialog had wrong format (%d) or length (%d)\n"),
-			   selection_data->format,
-			   selection_data->length);
+			   gtk_selection_data_get_format (selection_data),
+			   gtk_selection_data_get_length (selection_data));
 		return;
         }
 
-	uris = g_uri_list_extract_uris ((const char *)selection_data->data);
+	uris = g_uri_list_extract_uris ((const char *)gtk_selection_data_get_data (selection_data));
 
 	if (!uris) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
@@ -1661,56 +1758,66 @@ entry_drag_data_received (GtkEditable      *entry,
 
 static void
 panel_run_dialog_setup_entry (PanelRunDialog *dialog,
-			      GladeXML       *gui)
+			      GtkBuilder     *gui)
 {
 	GdkScreen             *screen;
 	int                    width_request;
+	GtkWidget             *entry;
 	
-	dialog->gnome_entry = glade_xml_get_widget (gui, "gnome_entry");
-	dialog->gtk_entry   = glade_xml_get_widget (gui, "gtk_entry");
+	dialog->combobox = PANEL_GTK_BUILDER_GET (gui, "comboboxentry");
+
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
+	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+
+	gtk_combo_box_set_model (GTK_COMBO_BOX (dialog->combobox),
+				 _panel_run_get_recent_programs_list ());
+	gtk_combo_box_set_entry_text_column
+		(GTK_COMBO_BOX (dialog->combobox), 0);
 
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
 
         /* 1/4 the width of the first monitor should be a good value */
 	width_request = panel_multiscreen_width (screen, 0) / 4;
-	g_object_set (G_OBJECT (dialog->gnome_entry),
+	g_object_set (G_OBJECT (dialog->combobox),
 		      "width_request", width_request,
 		      NULL);
 
-        g_signal_connect (dialog->gtk_entry, "key-press-event",
+        g_signal_connect (entry, "key-press-event",
 			  G_CALLBACK (entry_event), dialog);
 			  
-        dialog->changed_id = g_signal_connect (dialog->gtk_entry, "changed",
-					       G_CALLBACK (entry_changed),
+        dialog->changed_id = g_signal_connect (dialog->combobox, "changed",
+					       G_CALLBACK (combobox_changed),
 					       dialog);
 
-	gtk_drag_dest_unset (dialog->gtk_entry);
+	gtk_drag_dest_unset (dialog->combobox);
 	
-	gtk_drag_dest_set (dialog->gtk_entry,
+	gtk_drag_dest_set (dialog->combobox,
 			   GTK_DEST_DEFAULT_MOTION|GTK_DEST_DEFAULT_HIGHLIGHT,
 			   NULL, 0,
 			   GDK_ACTION_COPY);
-	gtk_drag_dest_add_uri_targets (dialog->gtk_entry);
+	gtk_drag_dest_add_uri_targets (dialog->combobox);
 
-	g_signal_connect (dialog->gtk_entry, "drag_data_received",
+	g_signal_connect (dialog->combobox, "drag_data_received",
 			  G_CALLBACK (entry_drag_data_received), dialog);
 }
 
 static char *
 panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 {
-	GKeyFile   *key_file;
-	gboolean    exec = FALSE;
-	const char *text;
-	char       *name;
-	char       *disk;
-	char       *scheme;
-	char       *save_uri;
+	GKeyFile *key_file;
+	gboolean  exec = FALSE;
+	char     *text;
+	char     *name;
+	char     *disk;
+	char     *scheme;
+	char     *save_uri;
 
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry));
+        text = g_strdup (panel_run_dialog_get_combo_text (dialog));
 	
-	if (!text || !text [0])
+	if (!text || !text [0]) {
+		g_free (text);
 		return NULL;
+	}
 		
 	key_file = panel_key_file_new_desktop ();
 	disk = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
@@ -1765,6 +1872,7 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 	g_key_file_free (key_file);
 	g_free (disk);
 	g_free (name);
+	g_free (text);
 
 	return save_uri;
 }
@@ -1786,16 +1894,15 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 
 	if (uri) {
 		gtk_selection_data_set (selection_data,
-					selection_data->target, 8,
+					gtk_selection_data_get_target (selection_data), 8,
 					(unsigned char *) uri, strlen (uri));
 		g_free (uri);
 	}
 }
 
 static void
-panel_run_dialog_style_set (GtkWidget      *widget,
-			    GtkStyle       *prev_style,
-			    PanelRunDialog *dialog)
+panel_run_dialog_style_updated (GtkWidget      *widget,
+                                PanelRunDialog *dialog)
 {
   if (dialog->icon_path) {
 	  char *icon_path;
@@ -1822,12 +1929,12 @@ panel_run_dialog_screen_changed (GtkWidget      *widget,
 
 static void
 panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
-			       GladeXML       *gui)
+			       GtkBuilder     *gui)
 {
-	dialog->pixmap = glade_xml_get_widget (gui, "icon_pixmap");
+	dialog->pixmap = PANEL_GTK_BUILDER_GET (gui, "icon_pixmap");
 	
-	g_signal_connect (dialog->pixmap, "style-set",
-			  G_CALLBACK (panel_run_dialog_style_set),
+	g_signal_connect (dialog->pixmap, "style-updated",
+			  G_CALLBACK (panel_run_dialog_style_updated),
 			  dialog);
 	g_signal_connect (dialog->pixmap, "screen-changed",
 			  G_CALLBACK (panel_run_dialog_screen_changed),
@@ -1839,15 +1946,15 @@ panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
 }
 
 static PanelRunDialog *
-panel_run_dialog_new (GdkScreen *screen,
-		      GladeXML  *gui,
+panel_run_dialog_new (GdkScreen  *screen,
+		      GtkBuilder *gui,
 		      guint32    activate_time)
 {
 	PanelRunDialog *dialog;
 
 	dialog = g_new0 (PanelRunDialog, 1);
 
-	dialog->run_dialog = glade_xml_get_widget (gui, "panel_run_dialog");
+	dialog->run_dialog = PANEL_GTK_BUILDER_GET (gui, "panel_run_dialog");
 	
 	g_signal_connect_swapped (dialog->run_dialog, "response",
 				  G_CALLBACK (panel_run_dialog_response), dialog);
@@ -1855,8 +1962,8 @@ panel_run_dialog_new (GdkScreen *screen,
 	g_signal_connect_swapped (dialog->run_dialog, "destroy",
 				  G_CALLBACK (panel_run_dialog_destroy), dialog);
 
-	dialog->run_button = glade_xml_get_widget (gui, "run_button");
-	dialog->terminal_checkbox = glade_xml_get_widget (gui, "terminal_checkbox");
+	dialog->run_button = PANEL_GTK_BUILDER_GET (gui, "run_button");
+	dialog->terminal_checkbox = PANEL_GTK_BUILDER_GET (gui, "terminal_checkbox");
 	
 	panel_run_dialog_setup_pixmap        (dialog, gui);
 	panel_run_dialog_setup_entry         (dialog, gui);
@@ -1875,9 +1982,9 @@ panel_run_dialog_new (GdkScreen *screen,
 
 	gtk_window_set_screen (GTK_WINDOW (dialog->run_dialog), screen);
 
-	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_grab_focus (dialog->combobox);
 	gtk_widget_realize (dialog->run_dialog);
-	gdk_x11_window_set_user_time (dialog->run_dialog->window,
+	gdk_x11_window_set_user_time (gtk_widget_get_window (dialog->run_dialog),
 				      activate_time);
 	gtk_widget_show (dialog->run_dialog);
 	
@@ -1888,7 +1995,7 @@ static void
 panel_run_dialog_disconnect_pixmap (PanelRunDialog *dialog)
 {
 	g_signal_handlers_disconnect_by_func (dialog->pixmap,
-			                      G_CALLBACK (panel_run_dialog_style_set),
+			                      G_CALLBACK (panel_run_dialog_style_updated),
 			                      dialog);
 	g_signal_handlers_disconnect_by_func (dialog->pixmap,
 			                      G_CALLBACK (panel_run_dialog_screen_changed),
@@ -1906,7 +2013,8 @@ void
 panel_run_dialog_present (GdkScreen *screen,
 			  guint32    activate_time)
 {
-	GladeXML *gui;
+	GtkBuilder *gui;
+	GError     *error;
 
 	if (panel_lockdown_get_disable_command_line ())
 		return;
@@ -1915,23 +2023,32 @@ panel_run_dialog_present (GdkScreen *screen,
 		gtk_window_set_screen (GTK_WINDOW (static_dialog->run_dialog), screen);
 		gtk_window_present_with_time (GTK_WINDOW (static_dialog->run_dialog),
 					      activate_time);
-		gtk_widget_grab_focus (static_dialog->gtk_entry);
+		gtk_widget_grab_focus (static_dialog->combobox);
 		return;
 	}
 
-	gui = glade_xml_new (GLADEDIR "/panel-run-dialog.glade",
-			     "panel_run_dialog",
-			     NULL);
-	if (!gui) {
+	gui = gtk_builder_new ();
+	gtk_builder_set_translation_domain (gui, GETTEXT_PACKAGE);
+
+	error = NULL;
+	gtk_builder_add_from_file (gui,
+				   BUILDERDIR "/panel-run-dialog.ui",
+				   &error);
+
+        if (error) {
 		char *secondary;
 
-		secondary = g_strdup_printf (_("Unable to load file '%s'."),
-					     GLADEDIR"/panel-run-dialog.glade");
+		secondary = g_strdup_printf (_("Unable to load file '%s': %s."),
+					     BUILDERDIR"/panel-run-dialog.ui",
+					     error->message);
 		panel_error_dialog (NULL, screen, "cannot_display_run_dialog",
 				    TRUE,
 				    _("Could not display run dialog"),
 				      secondary);
 		g_free (secondary);
+		g_error_free (error);
+		g_object_unref (gui);
+
 		return;
 	}
 
